import { createContext, useState, useEffect, createElement, useContext, cloneElement, useRef, Children } from 'react';
import { createPortal } from 'react-dom';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

function __generator(thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
}

var POPUP_ROOT_NAME = "popup-root";

function createRootElement() {
    var popupRootEl;
    var existingPopupRootEl = document.getElementById(POPUP_ROOT_NAME);
    if (existingPopupRootEl) {
        popupRootEl = existingPopupRootEl;
    }
    else {
        popupRootEl = document.createElement("div");
        popupRootEl.id = POPUP_ROOT_NAME;
        popupRootEl.style.cssText =
            "position: absolute; top: 0; left: 0; height: 0; width: 0;";
        document.body.appendChild(popupRootEl);
    }
    return popupRootEl;
}

function createReactPopup() {
    var popupRootEl = createRootElement();
    /**
     * Context for a Popup
     * -------------------------------------------------------------------
     */
    var popupContext = createContext({
        active: false,
        position: "trigger-bottom-left",
        setPopupState: function () { },
        triggerClientRect: null,
        onClose: function () { },
        onTrigger: function () { }
    });
    /**
     * Popup Trigger
     * -------------------------------------------------------------------
     */
    function PopupTrigger(_a) {
        var _b = _a.children, children = _b === void 0 ? createElement("button", null, "Trigger") : _b;
        var ctx = useContext(popupContext);
        function handleClick(e) {
            return __awaiter(this, void 0, void 0, function () {
                var triggerClientRect, node, _a;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            triggerClientRect = null;
                            if (e && e.currentTarget) {
                                node = e.target;
                                triggerClientRect = node.getBoundingClientRect();
                            }
                            _a = ctx.setPopupState;
                            if (!_a) return [3 /*break*/, 2];
                            return [4 /*yield*/, ctx.setPopupState(__assign(__assign({}, ctx), { active: true, triggerClientRect: triggerClientRect }))];
                        case 1:
                            _a = (_b.sent());
                            _b.label = 2;
                        case 2:
                            ctx.onTrigger();
                            return [2 /*return*/];
                    }
                });
            });
        }
        return cloneElement(children, { onClick: handleClick });
    }
    /**
     * Popup Content
     * -------------------------------------------------------------------
     */
    function PopupContent(_a) {
        var _b = _a.children, children = _b === void 0 ? createElement("p", null, "Content") : _b;
        var ref = useRef(null);
        var ctx = useContext(popupContext);
        var _c = useState(null), coords = _c[0], setCoords = _c[1];
        function updateCoords(clientRect) {
            var clRect = clientRect;
            var left = 0 + window.scrollX;
            var top = 0 + window.scrollY;
            switch (ctx.position) {
                case "trigger-top-left":
                    left += clRect.left;
                    top += clRect.top - ref.current.clientHeight;
                    break;
                case "trigger-top-right":
                    left += clRect.right - ref.current.clientWidth;
                    top += clRect.top - ref.current.clientHeight;
                    break;
                case "trigger-bottom-right":
                    left += clRect.right - ref.current.clientWidth;
                    top += clRect.top + clRect.height;
                    break;
                case "trigger-bottom-left":
                default:
                    left += clRect.left;
                    top += clRect.top + clRect.height;
            }
            setCoords({ left: left, top: top });
        }
        function handleClickOutside(e) {
            return __awaiter(this, void 0, void 0, function () {
                var _a;
                return __generator(this, function (_b) {
                    switch (_b.label) {
                        case 0:
                            if (!ref.current || ref.current.contains(e.target))
                                return [2 /*return*/];
                            _a = ctx.setPopupState;
                            if (!_a) return [3 /*break*/, 2];
                            return [4 /*yield*/, ctx.setPopupState(__assign(__assign({}, ctx), { active: false }))];
                        case 1:
                            _a = (_b.sent());
                            _b.label = 2;
                        case 2:
                            setCoords(null);
                            if (ctx.active && ctx.triggerClientRect)
                                ctx.onClose();
                            return [2 /*return*/];
                    }
                });
            });
        }
        useEffect(function () {
            if (ctx.active && ctx.triggerClientRect)
                updateCoords(ctx.triggerClientRect);
            document.addEventListener("mousedown", handleClickOutside);
            return function () {
                document.removeEventListener("mousedown", handleClickOutside);
            };
        }, [ctx.active, ctx.position]);
        return ctx.active
            ? createPortal(Children.map(children, function (child) {
                return cloneElement(child, {
                    ref: ref,
                    style: __assign(__assign(__assign({}, child.props.style), { position: "relative", visibility: ctx.active && coords != null ? "visible" : "hidden" }), coords)
                });
            }), popupRootEl)
            : null;
    }
    /**
     * Popup
     * -------------------------------------------------------------------
     */
    function Popup(_a) {
        var _this = this;
        var _b = _a.children, children = _b === void 0 ? function () { return null; } : _b, _c = _a.position, position = _c === void 0 ? "trigger-top-left" : _c, _d = _a.active, active = _d === void 0 ? false : _d, _e = _a.onClose, onClose = _e === void 0 ? function () { } : _e, _f = _a.onTrigger, onTrigger = _f === void 0 ? function () { } : _f;
        // Create a Popup state to pass as context
        var _g = useState({
            position: position,
            active: active,
            onClose: onClose,
            onTrigger: onTrigger
        }), popupState = _g[0], setPopupState = _g[1];
        // Re render if a specified prop changes
        useEffect(function () {
            setPopupState(__assign(__assign({}, popupState), { position: position }));
        }, [position]);
        var value = __assign(__assign({}, popupState), { setPopupState: setPopupState });
        // Return `children` props as function and share state
        return (createElement(popupContext.Provider, { value: value }, children(__assign(__assign({}, popupState), { close: function () { return __awaiter(_this, void 0, void 0, function () {
                return __generator(this, function (_a) {
                    switch (_a.label) {
                        case 0: return [4 /*yield*/, setPopupState(__assign(__assign({}, popupState), { active: false }))];
                        case 1:
                            _a.sent();
                            popupState.onClose();
                            return [2 /*return*/];
                    }
                });
            }); } }))));
    }
    Popup.Trigger = PopupTrigger;
    Popup.Content = PopupContent;
    return Popup;
}

export default createReactPopup;
//# sourceMappingURL=index.es.js.map
